from drf_spectacular.openapi import AutoSchema as BaseAutoSchema
from rest_framework.schemas.openapi import AutoSchema as DRFAutoSchema


class AutoSchema(BaseAutoSchema):

    def get_operation_id(self):
        """
        Spectacular generates operation IDs from URLs with the action as the last item.
        For example, `POST /worker/{id}/versions/` for a list API becomes `worker_versions_create`.
        However, we want `CreateWorkerVersion`, without a S, with worker_version coming from a
        Model, a Serializer or an APIView name; DRF's original logic.
        To do so, we actually call DRF's original logic.
        """
        schema = DRFAutoSchema()
        schema.view = self.view
        return schema.get_operation_id(self.path, self.method)

    def get_tags(self):
        """
        By default, Spectacular picks the first 'subfolder' in URLs as the default tag,
        for example /api/v1/corpus/<id> picks ['corpus'] as tags.
        We want to specify tags manually, so we return nothing instead.
        """
        return []

    def get_operation(self, *args, **kwargs):
        """
        Add an `x-paginated` property to each operation to allow the API client to tell whether
        or not to paginate when someone uses the pagination helper.
        When either _is_list_view() or _get_paginator() return False or None respectively,
        Spectacular does not include any pagination parameters.
        """
        operation = super().get_operation(*args, **kwargs)
        operation["x-paginated"] = self._is_list_view() and self._get_paginator() is not None
        return operation
